import josx.platform.rcx.*;

public abstract class Robot implements ButtonListener {
	// Steuerungskonstanten
	public final byte FORWARD = 1;
	public final byte BACKWARD = 2;
	public final byte LEFT = 3;
	public final byte RIGHT = 4;
	public final byte FLOAT = 5;
	public final byte STOP = 6;
	public final byte CIRCUIT_FORWARD = 7;
	public final byte CIRCUIT_LEFT = 8;
	public final byte CIRCUIT_RIGHT = 9;
	
	// Motoren-Power-Konstanten
	public final byte ROTATION_POWER = 5;
	public final byte DRIVE_POWER = 7;

	// alive ist wahr, wenn der Roboter "lebt"
	private boolean alive = true;
	
	// Roboter stirbt, wenn Button gedrueckt wird
	public void buttonPressed(Button b) {
		alive = false;
	}

	// Methode wird vom Interface verlangt,
	// wird aber nicht verwendet
	public void buttonReleased(Button b) {
	}

	// Methode zum Pr|fen, ob Roboter "lebt"
	public boolean isAlive() {
		return alive;
	}

	// fuehrt die Bewegung aus, die durch die uebergebene
	// Steuerungskonstante repraesentiert wird
	public void move(byte action) {
		if (action == LEFT) {
			motorsPower(ROTATION_POWER);
			Motor.C.forward();
			Motor.A.flt();
			pause(150);
			Motor.A.backward();
		}

		if (action == RIGHT) {
			motorsPower(ROTATION_POWER);
			Motor.A.forward();
			Motor.C.flt();
			pause(150);
			Motor.C.backward();
		}

		if (action == FORWARD) {
			motorsPower(DRIVE_POWER);
			Motor.A.forward();
			Motor.C.forward();
		}

		if (action == BACKWARD) {
			motorsPower(DRIVE_POWER);
			Motor.A.stop();
			Motor.C.stop();
			Motor.A.backward();
			Motor.C.backward();
		}

		if (action == FLOAT) {
			Motor.A.flt();
			Motor.C.flt();
			pause(100);
		}
		
		if (action == STOP) {
			Motor.A.stop();
			Motor.C.stop();
		}
		
		if (action == CIRCUIT_LEFT) {
			motorsPower(ROTATION_POWER);
			Motor.A.backward();
			Motor.C.forward();
		}

		if (action == CIRCUIT_RIGHT) {
			motorsPower(ROTATION_POWER);
			Motor.A.forward();
			Motor.C.backward();
		}

		if (action == CIRCUIT_FORWARD) {
			motorsPower(DRIVE_POWER);
			Motor.A.forward();
			Motor.C.forward();
		}
}

	// Geschwindigkeit der beider Motoren setzen
	private void motorsPower(byte power) {
		Motor.A.setPower(power);
		Motor.C.setPower(power);
	}

	// Zeitverzoegerung (in ms)
	public void pause(int time) {
		try {
			Thread.sleep(time);
		} catch (Exception exception) {}
	}
}