import josx.platform.rcx.*;

public class SendingRobot extends Robot {
	// Helligkeitsstufen der beiden Sensoren
	private byte[] darkLight;

	// Sensoren-Konstanten zu Erhhung
	// der Lesbarkeit des Quellcodes
	private final byte S1 = 0;
	private final byte S3 = 2;
	
	public SendingRobot() {
		// groe Reichweite fr Datenbertragung
		Serial.setRangeLong();

		// PRGM durch den ButtonListener Robot
		// berwachen lassen, damit der Roboter
		// durch Drcken von PRGM "stirbt"
		Button.PRGM.addButtonListener(this);
		
		// Lichtsensoren initalisieren
		initSensor(S1);
		initSensor(S3);
		
		// normales dunkles Licht ermitteln
		setNormalDarkLight();
		
		// drive ist wahr, wenn der Roboter geradeaus fhrt
		boolean drive = false;
		
		// onS1 und onS2 sind wahr, wenn der
		// jeweilige Sensor auf der Linie ist
		boolean onS1, onS3;
		
		// solange der Roboter "lebt", Linie
		// entlangfahren und Bewegungsbefehle senden
		while (isAlive()) {
			onS1 = isOnLine(S1);
			onS3 = isOnLine(S3);

			// losfahren, wenn Roboter nicht fhrt
			// und beide Sensoren auf der Linie
			if (!drive && onS1 && onS3) {
				sendAction(FORWARD);
				move(FORWARD);
				drive = true;
			}
			
			// rotieren, wenn der Roboter fhrt und wenn
			// mind. ein Sensor die Linie verlassen hat
			if (drive && (!onS1 || !onS3)) {
				rotateUntilOnLine(onS1, onS3);
				drive = false;
			}
		}

		// Roboter "sterben" lassen
		sendAction(FLOAT);
		move(FLOAT);
		Sensor.S1.passivate();
		Sensor.S3.passivate();
	}

	// der Roboter dreht solange bis er wieder auf der Linie ist
	private void rotateUntilOnLine(boolean onS1, boolean onS3) {
		// nur Sensor 1 verliert Linie
		if (!onS1 && onS3) {
			sendAction(RIGHT);
			move(RIGHT);
			while (!isOnLine(S1) && isAlive());
		}

		// nur Sensor 3 verliert Linie
		if (onS1 && !onS3) {
			sendAction(LEFT);
			move(LEFT);
			while (!isOnLine(S3) && isAlive());
		}
		
		// zurckfahren, da weder Sensor 1 noch Sensor 3 auf
		// der Linie ist, damit mind. ein Sensor auf der Linie
		if (!onS1 && !onS3) {
			sendAction(BACKWARD);
			move(BACKWARD);
		}
	}

	// Methode zum Pruefen, ob ein Sensor auf der Linie ist
	private boolean isOnLine(int sensor) {
		return Sensor.SENSORS[sensor].readValue() <= darkLight[sensor];
	}

	// Methode zum Initialisieren der Lichtsensoren
	private void initSensor(int sensor) {
		Sensor.SENSORS[sensor].setTypeAndMode(
				SensorConstants.SENSOR_TYPE_LIGHT,
				SensorConstants.SENSOR_MODE_PCT);
		Sensor.SENSORS[sensor].setPreviousValue(0);
		Sensor.SENSORS[sensor].activate();
	}

	// normales dunkles Licht ermitteln
	private void setNormalDarkLight() {
		int sumS1 = 0, sumS3 = 0, dataCount = 1000;
		for (int i = 0; i < dataCount; i++) {
			sumS1 += Sensor.S1.readValue();
			sumS3 += Sensor.S3.readValue();
		}
		darkLight = new byte[3];
		darkLight[S1] = (byte) (sumS1 / dataCount + 5);
		darkLight[S3] = (byte) (sumS3 / dataCount + 5);
		Sound.twoBeeps();
	}

	// Bewegungsaktion senden
	private void sendAction(byte action) {
		byte[] packet = { (byte) 0xf7, (byte) 0x00 };
		packet[1] = action;
		Serial.sendPacket(packet, 0, 2);
	}
	
	public static void main(String[] args) {
		new SendingRobot();
	}
}